@@ -121,9 +121,13 @@ class Command(BaseCommand):
             return
 
         # Query filters for SEP6 and 24
+
         withdraw_filters = Q(
             status=Transaction.STATUS.pending_user_transfer_start,
-            kind=Transaction.KIND.withdrawal,
+            kind__in=[
+                Transaction.KIND.withdrawal,
+                getattr(Transaction.KIND, "withdrawal-exchange"),
+            ],
         )
         # Query filters for SEP31
         send_filters = Q(
@@ -204,12 +208,14 @@ class Command(BaseCommand):
                 op, op_result, transaction.asset
             )
             if maybe_payment_data:
+                if ops[idx].source:
+                    source = ops[idx].source.account_muxed or ops[idx].source.account_id
+                else:
+                    source = (
+                        horizon_tx.source.account_muxed or horizon_tx.source.account_id
+                    )
                 await cls._update_transaction_info(
-                    transaction,
-                    response["id"],
-                    response["paging_token"],
-                    getattr(ops[idx].source, "account_id", None)
-                    or horizon_tx.source.account_id,
+                    transaction, response["id"], response["paging_token"], source
                 )
                 matching_payment_data = maybe_payment_data
                 break
